package uy.edu.fing.chesstrack.modulovision.imgproc;

import org.opencv.core.CvType;
import org.opencv.core.Mat;
import org.opencv.core.MatOfPoint2f;
import org.opencv.core.Point;
import org.opencv.core.Rect;
import org.opencv.core.Size;
import org.opencv.imgproc.Imgproc;

import uy.edu.fing.chesstrack.modulovision.Calibracion;
import android.util.Log;

public class Homografia implements ImgProcInterface {

	private static final String TAG = "CHESSTRACK::Homografia";
	private final  Mat matrizTransformada;
	private final Rect rectPOI;
	private int media;
	private int offset;
	public Homografia() throws Exception {
		super();
		MatOfPoint2f vertices = Calibracion.getInstance().getVertices();
		if (vertices != null){
			Mat src = new Mat(4,1,CvType.CV_32FC2);
			Mat dst = new Mat(4,1,CvType.CV_32FC2);
			offset = (media/7);
			Point p1, p2, p3, p4;
			Point[] ret = getBestPoints(vertices, Calibracion.getInstance().getImagenCalibrada().size());
			Log.i(TAG, "coso= " + ret);
			p1 = ret[0];
			p2 = ret[1];
			p3 = ret[2];
			p4 = ret[3];
			double d17 = Math.sqrt(Math.pow((p1.x- p2.x),2) + Math.pow((p1.y- p2.y),2));
			double d749 = Math.sqrt(Math.pow((p2.x- p3.x),2) + Math.pow((p2.y- p3.y),2));
			double d143 = Math.sqrt(Math.pow((p3.x- p4.x),2) + Math.pow((p3.y- p4.y),2));
			double d4349 = Math.sqrt(Math.pow((p4.x- p1.x),2) + Math.pow((p4.y- p1.y),2));

			media = (int) Math.floor( Math.round((d17+d749+d143+d4349)/4));
			offset = 2*(media/7);

			src.put(0,0 ,(int)p1.x,(int)p1.y, (int)p2.x,(int)p2.y, (int)p3.x,(int)p3.y, (int)p4.x,(int)p4.y);
			dst.put(0,0, offset,offset ,offset+media,offset  , offset+media,offset+media, offset,offset+media );

			// EL offset es casi el tamaño de una celda
			rectPOI = new Rect(new Point(0,0), new Point(2*offset+media,2*offset+media) );
			Calibracion.getInstance().setRectPOI(rectPOI);
			matrizTransformada = Imgproc.getPerspectiveTransform(src,dst);
		}else{
			throw new Exception(TAG + "No se han calculado los vertices");
		}
	}
	/**
	 * Hace homografia y recorta la imagen
	 */
	@Override
	public Mat procesarImagen(Mat inputFrame) {

		Imgproc.warpPerspective(inputFrame,inputFrame, matrizTransformada, inputFrame.size(),(Imgproc.INTER_LINEAR |  Imgproc.CV_WARP_FILL_OUTLIERS));
		Mat subMat = inputFrame.submat(rectPOI);
		Mat tmp = Mat.zeros(inputFrame.size(), CvType.CV_8UC4);
		Mat matTMP = tmp.submat(rectPOI);
		subMat.copyTo(matTMP);
		return tmp;
	}

	private Point[] getBestPoints(MatOfPoint2f vertices, Size tamanio){
		Point[] ret = new Point[4];

		double d, dx ,dy;
		double betst_dist = Double.MAX_VALUE;
		Point aux = new Point(0,0);
		for(Point pt : vertices.toList()){

			d = Math.sqrt(Math.pow((pt.x- aux.x),2) + Math.pow((pt.y- aux.y),2));
			if (d < betst_dist){
				betst_dist = d;
				ret[0] = pt;
			}
		}

		betst_dist = Double.MAX_VALUE;
		aux = new Point(tamanio.width,0);
		for(Point pt : vertices.toList()){
			d = Math.sqrt(Math.pow((pt.x- aux.x),2) + Math.pow((pt.y- aux.y),2));
			if (d < betst_dist){
				betst_dist = d;
				ret[1] = pt;
			}
		}

		betst_dist = Double.MAX_VALUE;
		aux = new Point(tamanio.width,tamanio.height);
		for(Point pt : vertices.toList()){
			d = Math.sqrt(Math.pow((pt.x- aux.x),2) + Math.pow((pt.y- aux.y),2));
			if (d < betst_dist){
				betst_dist = d;
				ret[2] = pt;
			}
		}

		betst_dist = Double.MAX_VALUE;
		aux = new Point(0,tamanio.height);
		for(Point pt : vertices.toList()){
			d = Math.sqrt(Math.pow((pt.x- aux.x),2) + Math.pow((pt.y- aux.y),2));
			if (d < betst_dist){
				betst_dist = d;
				ret[3] = pt;
			}
		}
		return ret;
	}

}
